/*
	tdsexample.c
	=============================================================================
		Blame        : Kevin Lyons <kevin@nol.org>
		Created      : 2004-02-08
		Last Updated : 2004-02-09 (kevin)
		Copyright(c) 2005 Kevin Lyons
	=============================================================================
	Description
	-----------
		Example of how to connect to and query from an MSSQL/Sybase server using 
	FreeTDS's libdb DB abstraction layer.

		This example is assumeing you're selecting two string/char/varchar records
	from a single table. Feel free to alter this as you see fit.
	=============================================================================
*/

#include <stdio.h>
#include <stdlib.h>

#include <sybdb.h>	/* NOTE - this include MUST come before dblib.h! */
#include <dblib.h>  /* NOTE - this include MUST come after sybdb.h!  */

/* EDIT HERE -> define your server/user/pass */
#define SERVER "foo"
#define USER   "user"
#define PASS   "pass"


/* message/error  handling functions - more on these later in the code */
int my_msg_handle(DBPROCESS*, DBINT, int, int, char*, char*, char*, int);
int my_err_handle(DBPROCESS*, int, int, int, char *, char *);


/*
	MAIN
	----
		Process:
			- Perform initial database setup
			- Open connection to the database server
			- Run a sample query 
			- Print output
*/
int main(int argc, char **argv)
{
	LOGINREC  *login = NULL;	 /* our login handler struct                       */
	DBPROCESS *dbh   = NULL;   /* what acts as our database "handle" if you will */
	char       query[255];     /* text for our query                             */
	char       column1[256];   /* EDIT HERE -> assume column 1 is a varchar(255) */
	char       column2[11];    /* EDIT HERE -> assume column 2 is a char(10)     */
	int        i;              /* counter                                        */

	/* first off, start up FreeTDS with all the trimmings. Will print to stderr
	   should an error occur */
	dbinit();

	/* now register our error handling functions so that we can report on problems
	   should the not come through the normal pipelines */
	dberrhandle(my_err_handle);
	dbmsghandle(my_msg_handle);

	/* allocate memory for our login structure and initialize it  - my_err_handle
	   will catch any errors here */
	login = dblogin();
	DBSETLUSER(login, USER);
	DBSETLPWD(login, PASS);
	
	/* now actually open the connection to the database, again the built in error
	   handling will catch any errors here */ 
	dbh = dbopen(login, SERVER);

	/* EDIT HERE -> alter this accordingly to fit your query needs */
	sprintf(query, "select Column1, Column2 from MyTable");

	/* and prep the query - any errors we might have get handled by the registerd
	   error/message handlers */
	dbcmd(dbh, query);

	/* and execute the query - once again, any errors we might have get handled
	   by the registered error/message handlers */
	dbsqlexec(dbh);

	/* check and make sure everything worked ok... */
	if (dbresults(dbh) != SUCCEED) {
		fprintf(stderr, "No result set found for '%s'!", query);
		fflush(stderr);
		exit(1);
	}

	/* print out the column titles */
	for(i = 1; i <= dbnumcols(dbh); i++) { printf("%s ", dbcolname(dbh, i)); }
	printf("\n");

	/* and here is where you'll need to edit carefully - make sure that you have
	   one dbbind entry for each column and that each type is correct or you may
		 end up with some serious memory issues here... */
	dbbind(dbh, 1, STRINGBIND, -1, (BYTE*)column1);
	dbbind(dbh, 2, STRINGBIND, -1, (BYTE*)column2);

	/* then, finally, print out the results */
	while (dbnextrow(dbh) == REG_ROW) {
		printf("%s  %s\n", column1, column2);
		fflush(stdout);
	}

	/* exit nicely now! */
	dbexit();
	
	return(0);
}
	
/* 
	my_msg_handle
	--------------
		Purpose: Define a message handling function to be called by the DB API should
		         any sort of message, errorr or warning come across the wire from 
		         the database engine.

		Notes: This handles most types of common messages, but may not catch all
		       types of them. Consult your database specs for more detail on what
		       codes you may need to handle.
*/
int my_msg_handle(
  DBPROCESS *dbh,                  /* our database handle                      */
  DBINT      message_number,       /* message number coming from the db server */
  int        message_state,        /* message state coming from the db server  */
  int        message_severity,     /* message severity coming form the db serv */
  char      *message_text,         /* actual text of the message (may be null) */
  char      *server_name,          /* name of the db server (usually null)     */
  char      *procedure_name,       /* if a stored procedure, the name or null  */
  int        line_of_message       /* on a query error, the line of the error  */
)
{

	switch(message_number) {
		case 5701: /* context change - ignore  */
		case 5703: /* language change - ignore */
		case 5704: /* charset change - ignore  */
			return 0;
			break;
		default:
			break;
	}

	if ( (message_severity > 0) && (message_number > 0) ) {
		/* if we have a real live message/error print the sucker! */
		fprintf(stderr, "Message   : %d\n", (int)message_number);
		fprintf(stderr, "Level     : %d\n", (int)message_severity);
		fprintf(stderr, "State     : %d\n", (int)message_state);	
		if ( (procedure_name != NULL) && (procedure_name[0] != 0) ) {
			fprintf(stderr, "Procedure : %s\n", procedure_name);
		}
		if ( (message_text != NULL) && (message_text[0] != 0) ) {
			fprintf(stderr, "Text      : %s\n", message_text);
		}
		if (line_of_message > 0) { 
			fprintf(stderr, "Line      : %d\n", line_of_message);
		}
	} else if ( (message_severity == 0) || (message_number == 0) ) {
		/* anything here will just be normal info messages we can ignore or not */
		if ( (message_text != NULL) && (message_text[0] != 0) ) {
			fprintf(stderr, "Informational Message: %s\n", message_text);
		}
	}
	fflush(stderr);

	return 0;
}
	

/*
	my_err_handle
	-------------
		Purpose: Define an error handling function to be called by the DB API should
		         any sort of internal library error occur.

		Notes: This handles most types of common errors, and the most notable one
		       you will likely see is "Attempt to bind to unknown column". This will
		       happen if you have some memory issues in your dbbinds (ie call to
		       dbbind on a NULL or unalloc'd string)
*/
int my_err_handle(
	DBPROCESS *dbh,            /* our database handle                         */
	int        error_severity, /* the severity of the error                   */
	int        dblib_number,   /* the DB API assigned error number            */
	int        oerror_number,  /* if an OpenClient error, the OC error number */
	char      *error_text,     /* text of the error message                   */
	char      *oerror_text     /* if an OpenClient error, the OC error text   */
)
{
	if (dblib_number == SYBESMSG) {
		/* if the error is from the database server side, pass through to our 
		   message handler function */
		return INT_CANCEL;
	}

	/* otherwise, we have an internal error (and possible seg fault!) */
	fprintf(stderr, "A libdb error has occurred\n");
	fprintf(stderr, "  Severity                : %d\n", error_severity);
	fprintf(stderr, "  DBlib Error      Number : %d\n", dblib_number);
	fprintf(stderr, "  OpenClient Error Number : %d\n", oerror_number);
	fprintf(stderr, "  Message Text            : %s\n", 
	                error_text ? error_text : "(null)");
	fprintf(stderr, "  OpenClient Text         : %s\n", 
	                oerror_text ? oerror_text : "(null)");
	fflush(stderr);

	if ( (dbh == NULL) || (DBDEAD(dbh)) ) {
		/* "If the dbprocess is dead or the dbproc is a
			 NULL pointer and we are not in the middle of logging in, then we need
		   to exit. We can't do anything from here on out anyway." - from 
			 in src/dblib/unittest/common.c in the FreeTDS distribution */
		if (dblib_number != SYBECOFL) { exit(1); }
	}

	return INT_CANCEL;
}
